﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using HeuristicLab.Services.Hive.DataAccess;
using HeuristicLab.Services.Hive;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using System.Text;
using System.Web.UI.DataVisualization.Charting;

public partial class Status : System.Web.UI.Page {
  protected void Page_Load(object sender, EventArgs e) {
    var dao = ServiceLocator.Instance.HiveDao;
    var onlineSlaves = dao.GetSlaves(x => x.SlaveState == SlaveState.Calculating || x.SlaveState == SlaveState.Idle);

    int currentlyAvailableCores = onlineSlaves.Sum(s => s.Cores.Value);
    int currentlyUsedCores = currentlyAvailableCores - onlineSlaves.Sum(s => s.FreeCores.Value);
    int currentlyJobsWaiting = ServiceLocator.Instance.HiveDao.GetJobs(x => x.State == JobState.Waiting).Count();

    this.availableCoresLabel.Text = currentlyAvailableCores.ToString();
    this.usedCoresLabel.Text = currentlyUsedCores.ToString();
    this.waitingJobsLabel.Text = currentlyJobsWaiting.ToString();

    slavesLabel.Text = string.Join(", ", onlineSlaves.Select(x => string.Format("{0} ({1} %)", x.Name, Math.Round(x.CpuUtilization, 2))));

    cpuUtilizationLabel.Text = onlineSlaves.Count() > 0 ? onlineSlaves.Average(s => s.CpuUtilization).ToString() : "0.0";

    HeuristicLab.Services.Hive.Common.DataTransfer.Statistics[] stats;
    if (daysDropDownList.SelectedValue == "All")
      stats = dao.GetStatistics(x => true).OrderByDescending(x => x.TimeStamp).ToArray();
    else
      stats = dao.GetStatistics(x => x.Timestamp >= DateTime.Now.Subtract(TimeSpan.FromDays(int.Parse(daysDropDownList.SelectedValue)))).OrderByDescending(x => x.TimeStamp).ToArray();


    for (int i = 0; i < stats.Length; i++) {
      var s = stats[i];

      var averageCpuUtilization = s.SlaveStatistics.Count() > 0 ? s.SlaveStatistics.Average(x => x.CpuUtilization) : 0.0;
      cpuUtilizationChart.Series[0].Points.Add(new DataPoint(s.TimeStamp.ToOADate(), averageCpuUtilization));

      var cores = s.SlaveStatistics.Sum(x => x.Cores);

      var usedCores = cores - s.SlaveStatistics.Sum(x => x.FreeCores);
      coresChart.Series[0].Points.AddXY(s.TimeStamp.ToOADate(), cores);
      coresChart.Series[1].Points.AddXY(s.TimeStamp.ToOADate(), usedCores);

      var memory = s.SlaveStatistics.Sum(x => x.Memory);
      var usedMemory = memory - s.SlaveStatistics.Sum(x => x.FreeMemory);
      memoryChart.Series[0].Points.AddXY(s.TimeStamp.ToOADate(), memory/1024);
      memoryChart.Series[1].Points.AddXY(s.TimeStamp.ToOADate(), usedMemory/1024);

      if (i > 0) {
        var execTime = new TimeSpan(s.UserStatistics.Sum(x => x.ExecutionTime.Ticks));
        var execTimePrev = new TimeSpan(stats[i - 1].UserStatistics.Sum(x => x.ExecutionTime.Ticks));
        var execTimeDifference = execTimePrev - execTime;

        var timeDifference = stats[i - 1].TimeStamp - s.TimeStamp; // the difference between statistic entries is not alway exactly 1 minute
        var speedup = execTimeDifference.TotalMinutes / timeDifference.TotalMinutes;
        speedupChartMinutes.Series[0].Points.AddXY(s.TimeStamp.ToOADate(), speedup);
      }
      if (i - 60 >= 0) {
        var execTime = new TimeSpan(s.UserStatistics.Sum(x => x.ExecutionTime.Ticks));
        var execTimePrev = new TimeSpan(stats[i - 60].UserStatistics.Sum(x => x.ExecutionTime.Ticks));
        var execTimeDifference = execTimePrev - execTime;

        var timeDifference = stats[i - 60].TimeStamp - s.TimeStamp; // the difference between statistic entries is not alway exactly 1 minute
        var speedup = execTimeDifference.TotalMinutes / timeDifference.TotalMinutes;
        speedupChartHours.Series[0].Points.AddXY(s.TimeStamp.ToOADate(), speedup);
      }
    }
  }

  protected void daysDropDownList_SelectedIndexChanged(object sender, EventArgs e) {

  }
}